#include <CL/sycl.hpp>
#include <iostream>

using namespace sycl;
using namespace std;

#define MEM_SIZE        92
#define COMPUTE_CYCLE   1024*1024*8
int main(int argc, char* argv[])
{
    // Run on Intel NUC 8th i5 (i5-8259U + Iris Plus Graphics 655)
    // Graphics driver: 30.0.101.1660

    queue q(default_selector{});    // Intel GPU
    //queue q(host_selector{});     // CPU Device for debugging ONLY

    // Device Name: Intel(R) Graphics [0x3ea5]
    cout << "Device Name: " << q.get_device().get_info<info::device::name>() << "\n";
    // 48 EU, 7 threads / EU = 336 threads
    // 128x32 bytes registers / thread

    uint32_t* globalMem = malloc_shared<uint32_t>(MEM_SIZE, q);
    q.parallel_for(range{ MEM_SIZE }, [=](id<1> idx) {
        
        // do some computation on the local array
        uint32_t localArray[MEM_SIZE];          // compiler should promote this array to registers
        memset(localArray, 0, sizeof(uint32_t) * MEM_SIZE);
        for (auto i = 1; i < MEM_SIZE; i++)
        {
            for (auto j = 0; j < COMPUTE_CYCLE; j++)
            {
                localArray[i] = j - (idx * i) - localArray[i - 1];
            }
        }
        // write back result to global memory
        globalMem[idx] = localArray[MEM_SIZE - 1];

    }).wait();

    for (auto i = 0; i < MEM_SIZE; i++)
    {
        cout << globalMem[i] << " ";
    }

    free(globalMem, q);
    return 0;
}